@@ -11,7 +11,7 @@
 %% The Original Code is Erlando.
 %%
 %% The Initial Developer of the Original Code is VMware, Inc.
-%% Copyright (c) 2011-2011 VMware, Inc.  All rights reserved.
+%% Copyright (c) 2011-2012 VMware, Inc.  All rights reserved.
 %%
 
 -module(monad).
@@ -39,8 +39,10 @@ join(Monad, X) ->
                  Y]).
 
 sequence(Monad, Xs) ->
-    lists:foldr(fun (X, Acc) ->
-                        do([Monad || E <- X,
-                                     Es <- Acc,
-                                     return([E|Es])])
-                end, Monad:return([]), Xs).
+    sequence(Monad, Xs, []).
+
+sequence(Monad, [], Acc) ->
+    do([Monad || return(lists:reverse(Acc))]);
+sequence(Monad, [X|Xs], Acc) ->
+    do([Monad || E <- X,
+                 sequence(Monad, Xs, [E|Acc])]).
