@@ -2,7 +2,7 @@
 %%---------------------------------------------------------------------------
 %% @author Tony Garnock-Jones <tonygarnockjones@gmail.com>
 %% @author LShift Ltd. <query@lshift.net>
-%% @copyright 2007-2010 Tony Garnock-Jones and 2007-2010 LShift Ltd.
+%% @copyright 2007-2010, 2011, 2012 Tony Garnock-Jones and 2007-2010 LShift Ltd.
 %% @license
 %%
 %% Permission is hereby granted, free of charge, to any person
@@ -53,7 +53,7 @@ terminate(_Reason, _State) ->
 
 %% @doc gen_server behaviour callback.
 code_change(_OldVsn, State, _Extra) ->
-    State.
+    {ok, State}.
 
 %% @doc gen_server behaviour callback.
 handle_call({lookup_service, Service}, _From, State) ->
@@ -64,6 +64,10 @@ handle_call({lookup_service, Service}, _From, State) ->
             {reply, ServiceRec, State}
     end;
 
+handle_call({register_service, ServiceDescription = #service{name = Name}}, _From, State) ->
+    ets:insert(?TABLE_NAME, {{service, Name}, ServiceDescription}),
+    {reply, ok, State};
+
 handle_call({register_service, Pid, ServiceDescription = #service{name = Name}}, _From, State) ->
     SD = ServiceDescription#service{handler = {pid, Pid}},
     erlang:monitor(process, Pid),
