@@ -30,7 +30,7 @@
 -export([init/1, handle_call/3, handle_cast/2, handle_info/2,
          terminate/2, code_change/3]).
 
--define(ROUTING_HEADER, <<"x-received-from">>).
+-import(rabbit_misc, [pget/2]).
 
 -record(state, {upstream,
                 connection,
@@ -68,7 +68,8 @@ list_routing_keys(XN) -> call(XN, list_routing_keys).
 start_link(Args) ->
     gen_server2:start_link(?MODULE, Args, [{timeout, infinity}]).
 
-init(Args = {_, XName}) ->
+init(Args = {Upstream, XName}) ->
+    rabbit_federation_status:report(Upstream, XName, starting),
     join(rabbit_federation_exchanges),
     join({rabbit_federation_exchange, XName}),
     gen_server2:cast(self(), maybe_go),
@@ -127,7 +128,9 @@ handle_info({#'basic.deliver'{routing_key  = Key,
               unacked             = Unacked}) ->
     Headers0 = extract_headers(Msg),
     %% TODO add user information here?
-    case should_forward(Headers0, MaxHops) of
+    %% We need to check should_forward/2 here in case the upstream
+    %% does not have federation and thus is using a fanout exchange.
+    case rabbit_federation_util:should_forward(Headers0, MaxHops) of
         true  -> {table, Info0} = rabbit_federation_upstream:to_table(Upstream),
                  Info = Info0 ++ [{<<"redelivered">>, bool, Redelivered}],
                  Headers = rabbit_basic:append_table_header(
@@ -152,7 +155,7 @@ handle_info({'DOWN', _Ref, process, Ch, shutdown},
 %% log it and die quietly.
 handle_info({'DOWN', _Ref, process, Ch, {shutdown, Reason}},
             State = #state{channel = Ch}) ->
-    connection_error({upstream_channel_down, Reason}, State);
+    connection_error(remote, {upstream_channel_down, Reason}, State);
 
 handle_info({'DOWN', _Ref, process, Ch, Reason},
             State = #state{channel = Ch}) ->
@@ -231,11 +234,17 @@ play_back_commands(State = #state{waiting_cmds = Waiting,
         true  -> State
     end.
 
+open_monitor(Params) ->
+    case open(Params) of
+        {ok, Conn, Ch} -> erlang:monitor(process, Ch),
+                          {ok, Conn, Ch};
+        E              -> E
+    end.
+
 open(Params) ->
     case amqp_connection:start(Params) of
         {ok, Conn} -> case amqp_connection:open_channel(Conn) of
-                          {ok, Ch} -> erlang:monitor(process, Ch),
-                                      {ok, Conn, Ch};
+                          {ok, Ch} -> {ok, Conn, Ch};
                           E        -> catch amqp_connection:close(Conn),
                                       E
                       end;
@@ -300,12 +309,12 @@ go(S0 = {not_started, {Upstream, DownXName =
     %% us to exit. We can therefore only trap exits when past that
     %% point. Bug 24372 may help us do something nicer.
     process_flag(trap_exit, true),
-    case open(rabbit_federation_util:local_params(DownVHost)) of
+    case open_monitor(rabbit_federation_util:local_params(DownVHost)) of
         {ok, DConn, DCh} ->
             #'confirm.select_ok'{} =
                amqp_channel:call(DCh, #'confirm.select'{}),
             amqp_channel:register_confirm_handler(DCh, self()),
-            case open(Upstream#upstream.params) of
+            case open_monitor(Upstream#upstream.params) of
                 {ok, Conn, Ch} ->
                     {Serial, Bindings} =
                         rabbit_misc:execute_mnesia_transaction(
@@ -320,43 +329,68 @@ go(S0 = {not_started, {Upstream, DownXName =
                     %% serial we will process. Since it compares larger than
                     %% any number we never process any commands. And we will
                     %% soon get told to stop anyway.
-                    State = ensure_upstream_bindings(
-                              consume_from_upstream_queue(
-                                #state{upstream              = Upstream,
-                                       connection            = Conn,
-                                       channel               = Ch,
-                                       next_serial           = Serial,
-                                       downstream_connection = DConn,
-                                       downstream_channel    = DCh,
+                    try
+                        State = ensure_upstream_bindings(
+                                  consume_from_upstream_queue(
+                                    #state{upstream              = Upstream,
+                                           connection            = Conn,
+                                           channel               = Ch,
+                                           next_serial           = Serial,
+                                           downstream_connection = DConn,
+                                           downstream_channel    = DCh,
                                        downstream_exchange   = DownXName}),
-                              Bindings),
-                    rabbit_log:info("Federation ~s connected to ~s~n",
-                                    [rabbit_misc:rs(DownXName),
-                                     rabbit_federation_upstream:to_string(
-                                       Upstream)]),
-                    {noreply, State};
-
+                                  Bindings),
+                        rabbit_log:info("Federation ~s connected to ~s~n",
+                                        [rabbit_misc:rs(DownXName),
+                                         rabbit_federation_upstream:to_string(
+                                           Upstream)]),
+                        Name = pget(name, amqp_connection:info(DConn, [name])),
+                        rabbit_federation_status:report(
+                          Upstream, DownXName, {running, Name}),
+                        {noreply, State}
+                    catch exit:E ->
+                            %% terminate/2 will not get this, as we
+                            %% have not put them in our state yet
+                            ensure_closed(DConn, DCh),
+                            ensure_closed(Conn, Ch),
+                            connection_error(remote, E, S0)
+                    end;
                 E ->
                     ensure_closed(DConn, DCh),
-                    connection_error(E, S0)
+                    connection_error(remote, E, S0)
             end;
         E ->
-            connection_error(E, S0)
+            connection_error(local, E, S0)
     end.
 
-connection_error(E, State = {not_started, {U, XName}}) ->
-    rabbit_log:info("Federation ~s failed to establish connection to ~s~n~p~n",
+connection_error(remote, E, State = {not_started, {U, XName}}) ->
+    rabbit_federation_status:report(U, XName, clean_reason(E)),
+    rabbit_log:warning("Federation ~s did not connect to ~s~n~p~n",
+                       [rabbit_misc:rs(XName),
+                        rabbit_federation_upstream:to_string(U), E]),
+    {stop, {shutdown, restart}, State};
+
+connection_error(remote, E, State = #state{upstream            = U,
+                                           downstream_exchange = XName}) ->
+    rabbit_federation_status:report(U, XName, clean_reason(E)),
+    rabbit_log:info("Federation ~s disconnected from ~s~n~p~n",
                     [rabbit_misc:rs(XName),
                      rabbit_federation_upstream:to_string(U), E]),
-    {stop, {shutdown, E}, State};
+    {stop, {shutdown, restart}, State};
 
-connection_error(E, State = #state{upstream            = U,
-                                   downstream_exchange = XName}) ->
-    rabbit_log:info("Federation ~s disconnected from ~s:~n~p~n",
-                    [rabbit_misc:rs(XName),
-                     rabbit_federation_upstream:to_string(U), E]),
-    {stop, {shutdown, E}, State}.
+connection_error(local, E, State = {not_started, {U, XName}}) ->
+    rabbit_federation_status:report(U, XName, clean_reason(E)),
+    rabbit_log:warning("Federation ~s did not connect locally~n~p~n",
+                       [rabbit_misc:rs(XName), E]),
+    {stop, {shutdown, restart}, State}.
 
+%% If we terminate due to a gen_server call exploding (almost
+%% certainly due to an amqp_channel:call() exploding) then we do not
+%% want to report the gen_server call in our status.
+clean_reason({E = {shutdown, _}, _}) -> E;
+clean_reason(E)                      -> E.
+
+%% local / disconnected never gets invoked, see handle_info({'DOWN', ...
 
 consume_from_upstream_queue(
   State = #state{upstream            = Upstream,
@@ -388,10 +422,7 @@ consume_from_upstream_queue(
     amqp_channel:call(Ch, #'queue.declare'{queue     = Q,
                                            durable   = true,
                                            arguments = Args}),
-    case Prefetch of
-        none -> ok;
-        _    -> amqp_channel:call(Ch, #'basic.qos'{prefetch_count = Prefetch})
-    end,
+    amqp_channel:call(Ch, #'basic.qos'{prefetch_count = Prefetch}),
     #'basic.consume_ok'{} =
         amqp_channel:subscribe(Ch, #'basic.consume'{queue  = Q,
                                                     no_ack = false}, self()),
@@ -411,15 +442,7 @@ ensure_upstream_bindings(State = #state{upstream            = Upstream,
                  <<"B">> -> <<"A">>
              end,
     IntXNameBin = upstream_exchange_name(XNameBin, VHost, DownXName, Suffix),
-    delete_upstream_exchange(Conn, IntXNameBin),
-    amqp_channel:call(
-      Ch, #'exchange.declare'{
-        exchange    = IntXNameBin,
-        type        = <<"fanout">>,
-        durable     = true,
-        internal    = true,
-        auto_delete = true,
-        arguments   = []}),
+    ensure_upstream_exchange(IntXNameBin, State),
     amqp_channel:call(Ch, #'queue.bind'{exchange = IntXNameBin, queue = Q}),
     State1 = State#state{internal_exchange = IntXNameBin},
     State2 = lists:foldl(fun add_binding/2, State1, Bindings),
@@ -429,6 +452,23 @@ ensure_upstream_bindings(State = #state{upstream            = Upstream,
     delete_upstream_exchange(Conn, OldIntXNameBin),
     State2.
 
+ensure_upstream_exchange(IntXNameBin,
+                         #state{upstream   = #upstream{max_hops = MaxHops,
+                                                       params   = Params},
+                                connection = Conn,
+                                channel    = Ch}) ->
+    delete_upstream_exchange(Conn, IntXNameBin),
+    Base = #'exchange.declare'{exchange    = IntXNameBin,
+                               durable     = true,
+                               internal    = true,
+                               auto_delete = true},
+    XFU = Base#'exchange.declare'{type      = <<"x-federation-upstream">>,
+                                  arguments = [{?MAX_HOPS_ARG, long, MaxHops}]},
+    Fan = Base#'exchange.declare'{type = <<"fanout">>},
+    disposable_connection_call(Params, XFU, fun(?COMMAND_INVALID, _Text) ->
+                                                    amqp_channel:call(Ch, Fan)
+                                            end).
+
 upstream_queue_name(XNameBin, VHost, #resource{name         = DownXNameBin,
                                                virtual_host = DownVHost}) ->
     Node = local_nodename(),
@@ -461,14 +501,31 @@ delete_upstream_exchange(Conn, XNameBin) ->
     disposable_channel_call(Conn, #'exchange.delete'{exchange = XNameBin}).
 
 disposable_channel_call(Conn, Method) ->
+    disposable_channel_call(Conn, Method, fun(_, _) -> ok end).
+
+disposable_channel_call(Conn, Method, ErrFun) ->
     {ok, Ch} = amqp_connection:open_channel(Conn),
     try
         amqp_channel:call(Ch, Method)
-    catch exit:{{shutdown, {server_initiated_close, _, _}}, _} ->
-            ok
+    catch exit:{{shutdown, {server_initiated_close, Code, Text}}, _} ->
+            ErrFun(Code, Text)
     end,
     ensure_closed(Ch).
 
+disposable_connection_call(Params, Method, ErrFun) ->
+    case open(Params) of
+        {ok, Conn, Ch} ->
+            try
+                amqp_channel:call(Ch, Method)
+            catch exit:{{shutdown, {connection_closing,
+                                    {server_initiated_close, Code, Txt}}}, _} ->
+                    ErrFun(Code, Txt)
+            end,
+            ensure_closed(Conn, Ch);
+        E ->
+            E
+    end.
+
 ensure_closed(Conn, Ch) ->
     ensure_closed(Ch),
     catch amqp_connection:close(Conn).
@@ -492,14 +549,6 @@ remove_delivery_tags(Seq, true, Unacked) ->
                  end
     end.
 
-should_forward(undefined, _MaxHops) ->
-    true;
-should_forward(Headers, MaxHops) ->
-    case rabbit_misc:table_lookup(Headers, ?ROUTING_HEADER) of
-        undefined  -> true;
-        {array, A} -> length(A) < MaxHops
-    end.
-
 extract_headers(#amqp_msg{props = #'P_basic'{headers = Headers}}) ->
     Headers.
 
