@@ -96,7 +96,8 @@ from_props_connection(Upst, ConnName, Conn, DefaultXNameBin, DefaultVHost) ->
                 XNameBin = pget_bin(exchange, Upst, DefaultXNameBin),
                 #upstream{params          = set_extra_params(Params, Conn),
                           exchange        = XNameBin,
-                          prefetch_count  = pget(prefetch_count,  Conn, none),
+                          prefetch_count  = pget(prefetch_count,  Conn,
+                                                 ?DEFAULT_PREFETCH),
                           reconnect_delay = pget(reconnect_delay, Conn, 1),
                           max_hops        = pget(max_hops,        Upst, 1),
                           expires         = pget(expires,         Conn, none),
