@@ -20,6 +20,8 @@
 -include_lib("eunit/include/eunit.hrl").
 -include_lib("amqp_client/include/amqp_client.hrl").
 
+-import(rabbit_misc, [pget/2]).
+
 -define(UPSTREAM_DOWNSTREAM, [x(<<"upstream">>),
                               fed(<<"downstream">>, <<"upstream">>)]).
 
@@ -272,12 +274,32 @@ max_hops() ->
     stop_other_node(?COTTONTAIL),
     ok.
 
+upstream_has_no_federation_test_() ->
+    {timeout, 60, fun upstream_has_no_federation/0}.
+
+upstream_has_no_federation() ->
+    with_ch(
+      fun (Downstream) ->
+              Upstream = start_other_node(
+                           ?HARE, "hare-no-federation", "no_plugins"),
+              declare_exchange(Upstream, x(<<"upstream">>)),
+              declare_exchange(Downstream,
+                               fed(<<"downstream">>, <<"upstream5673">>)),
+              Q = bind_queue(Downstream, <<"downstream">>, <<"routing">>),
+              publish(Upstream, <<"upstream">>, <<"routing">>, <<"HELLO">>),
+              expect(Downstream, Q, [<<"HELLO">>]),
+              delete_exchange(Downstream, <<"downstream">>),
+              delete_exchange(Upstream, <<"upstream">>),
+              stop_other_node(?HARE)
+      end, []).
+
 %%----------------------------------------------------------------------------
 
 with_ch(Fun, Xs) ->
     {ok, Conn} = amqp_connection:start(#amqp_params_network{}),
     {ok, Ch} = amqp_connection:open_channel(Conn),
     declare_all(Ch, Xs),
+    assert_status(Xs),
     Fun(Ch),
     delete_all(Ch, Xs),
     amqp_connection:close(Conn),
@@ -291,10 +313,15 @@ start_other_node({Name, Port}) ->
     start_other_node({Name, Port}, Name).
 
 start_other_node({Name, Port}, Config) ->
+    start_other_node({Name, Port}, Config,
+                     os:getenv("RABBITMQ_ENABLED_PLUGINS_FILE")).
+
+start_other_node({Name, Port}, Config, PluginsFile) ->
     %% ?assertCmd seems to hang if you background anything. Bah!
     Res = os:cmd("make -C " ++ plugin_dir() ++ " OTHER_NODE=" ++ Name ++
                      " OTHER_PORT=" ++ integer_to_list(Port) ++
                      " OTHER_CONFIG=" ++ Config ++
+                     " OTHER_PLUGINS=" ++ PluginsFile ++
                      " start-other-node ; echo $?"),
     LastLine = hd(lists:reverse(string:tokens(Res, "\n"))),
     ?assertEqual("0", LastLine),
@@ -421,3 +448,34 @@ test_args(Ch, Args) ->
                         type      = <<"x-federation">>,
                         arguments = Args}),
     delete_exchange(Ch, <<"test">>).
+
+assert_status(Xs) ->
+    Links = lists:append([links(X) || X <- Xs]),
+    Remaining = lists:foldl(fun assert_link_status/2,
+                            rabbit_federation_status:status(), Links),
+    ?assertEqual([], remove_configured_links(Remaining)),
+    ok.
+
+assert_link_status({DXNameBin, ConnectionName, UXNameBin}, Status) ->
+    {[_], Rest} = lists:partition(
+                    fun(St) ->
+                            pget(connection, St) =:= ConnectionName andalso
+                                pget(exchange, St) =:= DXNameBin andalso
+                                pget(upstream_exchange, St) =:= UXNameBin
+                    end, Status),
+    Rest.
+
+links(#'exchange.declare'{exchange  = Name,
+                          type      = <<"x-federation">>,
+                          arguments = Args}) ->
+    {longstr, Set} = rabbit_misc:table_lookup(Args,  <<"upstream-set">>),
+    {ok, Upstreams} = rabbit_federation_upstream:from_set(
+                        Set, rabbit_misc:r(<<"/">>, exchange, Name)),
+    [{Name, list_to_binary(U#upstream.connection_name), U#upstream.exchange} ||
+        U <- Upstreams];
+
+links(#'exchange.declare'{}) ->
+    [].
+
+remove_configured_links(Status) ->
+    [St || St <- Status, pget(exchange, St) =/= <<"down-conf">>].
