@@ -24,12 +24,15 @@
 
 -export([info/1, info/2]).
 
+-export([list_registry_plugins/1]).
+
 -include_lib("rabbit_common/include/rabbit.hrl").
 
 -define(REFRESH_RATIO, 5000).
 -define(KEYS, [os_pid, mem_ets, mem_binary, mem_proc, mem_proc_used,
                mem_atom, mem_atom_used, mem_code, fd_used, fd_total,
                sockets_used, sockets_total, mem_used, mem_limit, mem_alarm,
+               disk_free_limit, disk_free, disk_free_alarm,
                proc_used, proc_total, statistics_level,
                erlang_version, uptime, run_queue, processors, exchange_types,
                auth_mechanisms, applications]).
@@ -134,11 +137,17 @@ find_files_line(["  File " ++ Rest | _T]) ->
 find_files_line([_H | T]) ->
     find_files_line(T).
 
-get_memory_limit() ->
+-define(SAFE_CALL(Fun, NoProcFailResult),
     try
-        vm_memory_monitor:get_memory_limit()
-    catch exit:{noproc, _} -> memory_monitoring_disabled
-    end.
+        Fun
+    catch exit:{noproc, _} -> NoProcFailResult
+    end).
+
+get_disk_free_limit() -> ?SAFE_CALL(rabbit_disk_monitor:get_disk_free_limit(),
+                                    disk_free_monitoring_disabled).
+
+get_disk_free() -> ?SAFE_CALL(rabbit_disk_monitor:get_disk_free(),
+                              disk_free_monitoring_disabled).
 
 %%--------------------------------------------------------------------
 
@@ -146,25 +155,29 @@ infos(Items, State) -> [{Item, i(Item, State)} || Item <- Items].
 
 i(fd_used,  #state{fd_used  = FdUsed})  -> FdUsed;
 i(fd_total, #state{fd_total = FdTotal}) -> FdTotal;
-i(sockets_used,   _State) ->
+i(sockets_used,    _State) ->
     proplists:get_value(sockets_used, file_handle_cache:info([sockets_used]));
-i(sockets_total,  _State) ->
+i(sockets_total,   _State) ->
     proplists:get_value(sockets_limit, file_handle_cache:info([sockets_limit]));
-i(os_pid,         _State) -> list_to_binary(os:getpid());
-i(mem_ets,        _State) -> erlang:memory(ets);
-i(mem_binary,     _State) -> erlang:memory(binary);
-i(mem_proc,       _State) -> erlang:memory(processes);
-i(mem_proc_used,  _State) -> erlang:memory(processes_used);
-i(mem_atom,       _State) -> erlang:memory(atom);
-i(mem_atom_used,  _State) -> erlang:memory(atom_used);
-i(mem_code,       _State) -> erlang:memory(code);
-i(mem_used,       _State) -> erlang:memory(total);
-i(mem_limit,      _State) -> get_memory_limit();
-i(proc_used,      _State) -> erlang:system_info(process_count);
-i(proc_total,     _State) -> erlang:system_info(process_limit);
-i(erlang_version, _State) -> list_to_binary(erlang:system_info(otp_release));
-i(run_queue,      _State) -> erlang:statistics(run_queue);
-i(processors,     _State) -> erlang:system_info(logical_processors);
+i(os_pid,          _State) -> list_to_binary(os:getpid());
+i(mem_ets,         _State) -> erlang:memory(ets);
+i(mem_binary,      _State) -> erlang:memory(binary);
+i(mem_proc,        _State) -> erlang:memory(processes);
+i(mem_proc_used,   _State) -> erlang:memory(processes_used);
+i(mem_atom,        _State) -> erlang:memory(atom);
+i(mem_atom_used,   _State) -> erlang:memory(atom_used);
+i(mem_code,        _State) -> erlang:memory(code);
+i(mem_used,        _State) -> erlang:memory(total);
+i(mem_limit,       _State) -> vm_memory_monitor:get_memory_limit();
+i(mem_alarm,       _State) -> resource_alarm_set(memory);
+i(proc_used,       _State) -> erlang:system_info(process_count);
+i(proc_total,      _State) -> erlang:system_info(process_limit);
+i(erlang_version,  _State) -> list_to_binary(erlang:system_info(otp_release));
+i(run_queue,       _State) -> erlang:statistics(run_queue);
+i(processors,      _State) -> erlang:system_info(logical_processors);
+i(disk_free_limit, _State) -> get_disk_free_limit();
+i(disk_free,       _State) -> get_disk_free();
+i(disk_free_alarm, _State) -> resource_alarm_set(disk);
 i(uptime, _State) ->
     {Total, _} = erlang:statistics(wall_clock),
     Total;
@@ -180,9 +193,11 @@ i(auth_mechanisms, _State) ->
       fun (N) -> lists:member(list_to_atom(binary_to_list(N)), Mechanisms) end);
 i(applications, _State) ->
     [format_application(A) ||
-        A <- lists:keysort(1, application:which_applications(infinity))];
-i(mem_alarm, _State) -> lists:member({{vm_memory_high_watermark, node()}, []},
-                                     alarm_handler:get_alarms()).
+        A <- lists:keysort(1, application:which_applications(infinity))].
+
+resource_alarm_set(Source) ->
+    lists:member({{resource_limit, Source, node()},[]},
+                 alarm_handler:get_alarms()).
 
 list_registry_plugins(Type) ->
     list_registry_plugins(Type, fun(_) -> true end).
