@@ -70,6 +70,11 @@ HELP = {
       incoming network traffic until the memory usage drops below \
       the watermark.',
 
+    'disk-free-alarm':
+      'The disk free space alarm for this node has gone off. It will block \
+      incoming network traffic until the amount of free space exceeds \
+      the limit.',
+
     'message-get-requeue':
       '<p>Clicking "Get Message(s)" will consume messages from the queue. \
       If requeue is set the message will be re-added to the queue, \
@@ -170,9 +175,11 @@ HELP = {
         <dd>Rate at which basic.return is sent to publishers for undeliverable  messages published with the \'immediate\' flag set.</dd>\
       </dl>',
 
+    'disk-monitoring-no-watermark' : 'There is no disk space low watermark set. RabbitMQ will not take any action to avoid running out of disk space.',
+
     'foo': 'foo' // No comma.
 };
 
 function help(id) {
     show_popup('help', HELP[id]);
-}
\ No newline at end of file
+}
