@@ -272,7 +272,7 @@ function postprocess() {
             update_multifields();
         });
     $('.controls-appearance').change(function() {
-        var controls = $(this).attr('controls');
+        var controls = $(this).attr('controls-divs');
         if ($(this).val() == 'true') {
             $('#' + controls + '-yes').slideDown(100);
             $('#' + controls + '-no').slideUp(100);
@@ -644,6 +644,9 @@ function add_known_arguments(params) {
 
 function check_password(params) {
     if (params['password'] != undefined) {
+        if (params['password'] == '') {
+            throw("Please specify a password.");
+        }
         if (params['password'] != params['password_confirm']) {
             throw("Passwords do not match.");
         }
