@@ -11,7 +11,7 @@
 %%   The Original Code is RabbitMQ Management Plugin.
 %%
 %%   The Initial Developer of the Original Code is VMware, Inc.
-%%   Copyright (c) 2010-2011 VMware, Inc.  All rights reserved.
+%%   Copyright (c) 2010-2012 VMware, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mgmt_db).
@@ -314,9 +314,8 @@ handle_event(Event = #event{type = queue_deleted}, State) ->
     handle_deleted(queue_stats, Event, State);
 
 handle_event(#event{type = connection_created, props = Stats}, State) ->
-    Name = rabbit_mgmt_format:connection(Stats),
     handle_created(
-      connection_stats, [{name, Name} | proplists:delete(name, Stats)],
+      connection_stats, Stats,
       [{fun rabbit_mgmt_format:addr/1,         [address, peer_address]},
        {fun rabbit_mgmt_format:port/1,         [port, peer_port]},
        {fun rabbit_mgmt_format:protocol/1,     [protocol]},
@@ -331,14 +330,8 @@ handle_event(#event{type = connection_stats, props = Stats,
 handle_event(Event = #event{type = connection_closed}, State) ->
     handle_deleted(connection_stats, Event, State);
 
-handle_event(#event{type = channel_created, props = Stats},
-             State = #state{tables = Tables}) ->
-    ConnTable = orddict:fetch(connection_stats, Tables),
-    Conn = lookup_element(ConnTable, {id(pget(connection, Stats)), create}),
-    Name = rabbit_mgmt_format:print("~s:~w",
-                                    [pget(name,   Conn),
-                                     pget(number, Stats)]),
-    handle_created(channel_stats, [{name, Name}|Stats], [], State);
+handle_event(#event{type = channel_created, props = Stats}, State) ->
+    handle_created(channel_stats, Stats, [], State);
 
 handle_event(#event{type = channel_stats, props = Stats, timestamp = Timestamp},
              State) ->
