@@ -11,7 +11,7 @@
 %%   The Original Code is RabbitMQ Management Plugin.
 %%
 %%   The Initial Developer of the Original Code is VMware, Inc.
-%%   Copyright (c) 2010-2011 VMware, Inc.  All rights reserved.
+%%   Copyright (c) 2010-2012 VMware, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mgmt_format).
@@ -23,7 +23,7 @@
 -export([pack_binding_props/2, unpack_binding_props/1, tokenise/1]).
 -export([to_amqp_table/1, listener/1, properties/1, basic_properties/1]).
 -export([record/2, to_basic_properties/1]).
--export([connection/1, addr/1, port/1]).
+-export([addr/1, port/1]).
 
 -include_lib("rabbit_common/include/rabbit.hrl").
 -include_lib("rabbit_common/include/rabbit_framing.hrl").
@@ -129,7 +129,7 @@ timestamp(unknown) ->
     unknown;
 timestamp(Timestamp) ->
     {{Y, M, D}, {H, Min, S}} = calendar:now_to_local_time(Timestamp),
-    print("~w-~w-~w ~w:~w:~w", [Y, M, D, H, Min, S]).
+    print("~w-~2.2.0w-~2.2.0w ~w:~2.2.0w:~2.2.0w", [Y, M, D, H, Min, S]).
 
 resource(unknown) -> unknown;
 resource(Res)     -> resource(name, Res).
@@ -237,14 +237,6 @@ type_val(X)                   -> throw({error, {unhandled_type, X}}).
 url(Fmt, Vals) ->
     print(Fmt, [mochiweb_util:quote_plus(V) || V <- Vals]).
 
-connection(Props) ->
-    case proplists:get_value(name, Props, unknown) of
-        unknown -> print("~s:~w",
-                         [addr(proplists:get_value(peer_address, Props)),
-                          port(proplists:get_value(peer_port,    Props))]);
-        Name      -> Name
-    end.
-
 exchange(X) ->
     format(X, [{fun resource/1,   [name]},
                {fun amqp_table/1, [arguments]}]).
