@@ -11,7 +11,7 @@
 %%   The Original Code is RabbitMQ Management Plugin.
 %%
 %%   The Initial Developer of the Original Code is VMware, Inc.
-%%   Copyright (c) 2010-2011 VMware, Inc.  All rights reserved.
+%%   Copyright (c) 2010-2012 VMware, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mgmt_wm_user).
@@ -86,19 +86,22 @@ put_user(User) ->
 
 put_user(User, PWArg, PWFun) ->
     Username = pget(name, User),
-    Tags = case pget(tags, User) of
-               undefined -> case rabbit_mgmt_util:parse_bool(
-                                   pget(administrator, User)) of
-                                true  -> [administrator];
-                                false -> []
-                            end;
-               TagsS     -> [list_to_atom(string:strip(T)) ||
-                                T <- string:tokens(binary_to_list(TagsS), ",")]
+    Tags = case {pget(tags, User), pget(administrator, User)} of
+               {undefined, undefined} ->
+                   throw({error, tags_not_present});
+               {undefined, AdminS} ->
+                   case rabbit_mgmt_util:parse_bool(AdminS) of
+                       true  -> [administrator];
+                       false -> []
+                   end;
+               {TagsS, _} ->
+                   [list_to_atom(string:strip(T)) ||
+                       T <- string:tokens(binary_to_list(TagsS), ",")]
            end,
     case rabbit_auth_backend_internal:lookup_user(Username) of
         {error, not_found} ->
             rabbit_auth_backend_internal:add_user(
-              Username, rabbit_guid:binstring_guid("tmp"));
+              Username, rabbit_guid:binary(rabbit_guid:gen_secure(), "tmp"));
         _ ->
             ok
     end,
