@@ -11,7 +11,7 @@
 %%   The Original Code is RabbitMQ Management Console.
 %%
 %%   The Initial Developer of the Original Code is VMware, Inc.
-%%   Copyright (c) 2010-2011 VMware, Inc.  All rights reserved.
+%%   Copyright (c) 2010-2012 VMware, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mgmt_test_db).
@@ -193,6 +193,9 @@ test_channel_aggregation(Conn, Chan) ->
     X2 = <<"channel-aggregation-exch2">>,
     declare_exchange(Chan, X1),
     declare_exchange(Chan, X2),
+    Q = declare_queue(Chan),
+    bind_queue(Chan, X1, Q),
+    bind_queue(Chan, X2, Q),
     publish(Chan, X1, <<"">>, 10),
     publish(Chan, X2, <<"">>, 100),
 
@@ -266,8 +269,8 @@ find_conn_by_local_port(Port, Items) ->
 get_channel(C, Number) ->
     Port = local_port(C),
     hd(rabbit_mgmt_db:get_channels(
-         [list_to_binary("127.0.0.1:" ++ integer_to_list(Port) ++ ":" ++
-                             integer_to_list(Number))], full)).
+         [rabbit_mgmt_format:print(
+            "127.0.0.1:~w -> 127.0.0.1:5672 (~w)", [Port, Number])], full)).
 
 get_exchange(XName) ->
     X = rabbit_mgmt_wm_exchange:exchange(<<"/">>, XName),
