@@ -11,7 +11,7 @@
 %%   The Original Code is RabbitMQ Management Console.
 %%
 %%   The Initial Developer of the Original Code is VMware, Inc.
-%%   Copyright (c) 2010-2011 VMware, Inc.  All rights reserved.
+%%   Copyright (c) 2010-2012 VMware, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mgmt_test_http).
@@ -104,6 +104,17 @@ users_test() ->
     http_get("/users/myuser", ?NOT_FOUND),
     ok.
 
+users_legacy_administrator_test() ->
+    http_put("/users/myuser1", [{administrator, <<"true">>}], ?NO_CONTENT),
+    http_put("/users/myuser2", [{administrator, <<"false">>}], ?NO_CONTENT),
+    assert_item([{name, <<"myuser1">>}, {tags, <<"administrator">>}],
+                http_get("/users/myuser1")),
+    assert_item([{name, <<"myuser2">>}, {tags, <<"">>}],
+                http_get("/users/myuser2")),
+    http_delete("/users/myuser1", ?NO_CONTENT),
+    http_delete("/users/myuser2", ?NO_CONTENT),
+    ok.
+
 permissions_validation_test() ->
     Good = [{configure, <<".*">>}, {write, <<".*">>}, {read, <<".*">>}],
     http_put("/permissions/wrong/guest", Good, ?BAD_REQUEST),
@@ -178,7 +189,8 @@ connections_test() ->
     LocalPort = rabbit_mgmt_test_db:local_port(Conn),
     Path = binary_to_list(
              rabbit_mgmt_format:print(
-               "/connections/127.0.0.1%3A~w", [LocalPort])),
+               "/connections/127.0.0.1%3A~w%20->%20127.0.0.1%3A5672",
+               [LocalPort])),
     http_get(Path, ?OK),
     http_delete(Path, ?NO_CONTENT),
     %% TODO rabbit_reader:shutdown/2 returns before the connection is
@@ -473,10 +485,12 @@ get_conn(Username, Password) ->
     LocalPort = rabbit_mgmt_test_db:local_port(Conn),
     ConnPath = binary_to_list(
                  rabbit_mgmt_format:print(
-                   "/connections/127.0.0.1%3A~w", [LocalPort])),
+                   "/connections/127.0.0.1%3A~w%20->%20127.0.0.1%3A5672",
+                   [LocalPort])),
     ChPath = binary_to_list(
                rabbit_mgmt_format:print(
-                 "/channels/127.0.0.1%3A~w%3A1", [LocalPort])),
+                 "/channels/127.0.0.1%3A~w%20->%20127.0.0.1%3A5672%20(1)",
+                 [LocalPort])),
     {Conn, ConnPath, ChPath}.
 
 permissions_connection_channel_test() ->
