@@ -18,14 +18,14 @@
 
 -export([start_link/2]).
 -export([init/2]).
--export([conserve_memory/2]).
+-export([conserve_resources/2]).
 
 -include("rabbit_stomp.hrl").
 -include("rabbit_stomp_frame.hrl").
 -include_lib("amqp_client/include/amqp_client.hrl").
 
 -record(reader_state, {socket, parse_state, processor, state, iterations,
-                       conserve_memory}).
+                       conserve_resources, recv_outstanding}).
 
 %%----------------------------------------------------------------------------
 
@@ -47,13 +47,14 @@ init(SupPid, Configuration) ->
             try
                 mainloop(
                   control_throttle(
-                    register_memory_alarm(
-                      #reader_state{socket          = Sock,
-                                    parse_state     = ParseState,
-                                    processor       = ProcessorPid,
-                                    state           = running,
-                                    iterations      = 0,
-                                    conserve_memory = false})), 0),
+                    register_resource_alarm(
+                      #reader_state{socket             = Sock,
+                                    parse_state        = ParseState,
+                                    processor          = ProcessorPid,
+                                    state              = running,
+                                    iterations         = 0,
+                                    conserve_resources = false,
+                                    recv_outstanding   = false})), 0),
                 log(info, "closing STOMP connection ~p (~s)~n",
                     [self(), ConnStr])
             catch
@@ -66,19 +67,19 @@ init(SupPid, Configuration) ->
             done
     end.
 
-mainloop(State = #reader_state{socket = Sock}, ByteCount) ->
-    run_socket(State, ByteCount),
+mainloop(State0 = #reader_state{socket = Sock}, ByteCount) ->
+    State = run_socket(State0, ByteCount),
     receive
         {inet_async, Sock, _Ref, {ok, Data}} ->
-            process_received_bytes(Data, State);
+            process_received_bytes(Data, State#reader_state{recv_outstanding = false});
         {inet_async, _Sock, _Ref, {error, closed}} ->
             ok;
         {inet_async, _Sock, _Ref, {error, Reason}} ->
             throw({inet_error, Reason});
-        {conserve_memory, Conserve} ->
+        {conserve_resources, Conserve} ->
             mainloop(
               control_throttle(
-                State#reader_state{conserve_memory = Conserve}), ByteCount);
+                State#reader_state{conserve_resources = Conserve}), ByteCount);
         {bump_credit, Msg} ->
             credit_flow:handle_bump_msg(Msg),
             mainloop(control_throttle(State), ByteCount)
@@ -104,15 +105,15 @@ process_received_bytes(Bytes,
                                        state       = next_state(S, Frame)}))
     end.
 
-conserve_memory(Pid, Conserve) ->
-    Pid ! {conserve_memory, Conserve},
+conserve_resources(Pid, Conserve) ->
+    Pid ! {conserve_resources, Conserve},
     ok.
 
-register_memory_alarm(State) ->
-    rabbit_alarm:register(self(), {?MODULE, conserve_memory, []}), State.
+register_resource_alarm(State) ->
+    rabbit_alarm:register(self(), {?MODULE, conserve_resources, []}), State.
 
-control_throttle(State = #reader_state{state            = CS,
-                                       conserve_memory  = Mem}) ->
+control_throttle(State = #reader_state{state              = CS,
+                                       conserve_resources = Mem}) ->
     case {CS, Mem orelse credit_flow:blocked()} of
         {running,   true} -> State#reader_state{state = blocking};
         {blocking, false} -> State#reader_state{state = running};
@@ -125,11 +126,13 @@ next_state(blocking, #stomp_frame{command = "SEND"}) ->
 next_state(S, _) ->
     S.
 
-run_socket(#reader_state{state = blocked}, _ByteCount) ->
-    ok;
-run_socket(#reader_state{socket = Sock}, ByteCount) ->
+run_socket(State = #reader_state{state = blocked}, _ByteCount) ->
+    State;
+run_socket(State = #reader_state{recv_outstanding = true}, _ByteCount) ->
+    State;
+run_socket(State = #reader_state{socket = Sock}, ByteCount) ->
     rabbit_net:async_recv(Sock, ByteCount, infinity),
-    ok.
+    State#reader_state{recv_outstanding = true}.
 
 %%----------------------------------------------------------------------------
 
