@@ -61,7 +61,9 @@ message_headers_test() ->
 
     Delivery = #'basic.deliver'{
       consumer_tag = <<"Q_123">>,
-      delivery_tag = 123},
+      delivery_tag = 123,
+      exchange = <<"">>,
+      routing_key = <<"foo">>},
 
     Properties = #'P_basic'{
       headers          = [{<<"str">>, longstr, <<"foo">>},
@@ -74,8 +76,8 @@ message_headers_test() ->
       reply_to         = <<"something">>,
       message_id       = <<"M123">>},
 
-    Headers = rabbit_stomp_util:message_headers(Destination, SessionId,
-                                                Delivery, Properties),
+    Headers = rabbit_stomp_util:message_headers(SessionId, Delivery,
+                                                Properties),
 
     Expected = [
                 {"destination", Destination},
@@ -99,12 +101,14 @@ minimal_message_headers_with_no_custom_test() ->
 
     Delivery = #'basic.deliver'{
       consumer_tag = <<"Q_123">>,
-      delivery_tag = 123},
+      delivery_tag = 123,
+      exchange = <<"">>,
+      routing_key = <<"foo">>},
 
     Properties = #'P_basic'{},
 
-    Headers = rabbit_stomp_util:message_headers(Destination, SessionId,
-                                                Delivery, Properties),
+    Headers = rabbit_stomp_util:message_headers(SessionId, Delivery,
+                                                Properties),
     Expected = [
                 {"destination", Destination},
                 {"message-id", [<<"Q_123">>, "@@", SessionId, "@@", "123"]},
