@@ -14,15 +14,23 @@ def ensure_ssl_auth_user():
     rabbitmqctl(['set_permissions', user, '.*', '.*', '.*'])
 
 def enable_implicit_connect():
-    switch_implicit_connect('true', '[{login, "guest"}, {passcode, "guest"}]')
+    switch_config(implicit_connect='true', default_user='[{login, "guest"}, {passcode, "guest"}]')
 
 def disable_implicit_connect():
-    switch_implicit_connect('false', '[]')
+    switch_config(implicit_connect='false', default_user='[]')
 
-def switch_implicit_connect(implicit_connect, default_user):
-    cmd = 'application:set_env(rabbitmq_stomp,implicit_connect,' + implicit_connect + '),'
-    cmd += 'application:set_env(rabbitmq_stomp,default_user,' + default_user + '),'
-    cmd += 'application:stop(rabbitmq_stomp),'
+def enable_default_user():
+    switch_config(default_user='[{login, "guest"}, {passcode, "guest"}]')
+
+def disable_default_user():
+    switch_config(default_user='[]')
+
+def switch_config(implicit_connect='', default_user=''):
+    cmd = 'application:stop(rabbitmq_stomp),'
+    if implicit_connect:
+        cmd += 'application:set_env(rabbitmq_stomp,implicit_connect,' + implicit_connect + '),'
+    if default_user:
+        cmd += 'application:set_env(rabbitmq_stomp,default_user,' + default_user + '),'
     cmd += 'application:start(rabbitmq_stomp).'
     rabbitmqctl(['eval', cmd])
 
@@ -31,3 +39,4 @@ def rabbitmqctl(args):
     cmdline = [ctl, '-n', 'rabbit-test']
     cmdline.extend(args)
     subprocess.check_call(cmdline)
+
