@@ -327,7 +327,11 @@ status() ->
            [{vm_memory_high_watermark, {vm_memory_monitor,
                                         get_vm_memory_high_watermark, []}},
             {vm_memory_limit,          {vm_memory_monitor,
-                                        get_memory_limit, []}}]),
+                                        get_memory_limit, []}},
+            {disk_free_limit,          {rabbit_disk_monitor,
+                                        get_disk_free_limit, []}},
+            {disk_free,                {rabbit_disk_monitor,
+                                        get_disk_free, []}}]),
     S3 = rabbit_misc:with_exit_handler(
            fun () -> [] end,
            fun () -> [{file_descriptors, file_handle_cache:info()}] end),
@@ -343,10 +347,7 @@ status() ->
 is_running() -> is_running(node()).
 
 is_running(Node) ->
-    case rpc:call(Node, application, which_applications, [infinity]) of
-        {badrpc, _} -> false;
-        Apps        -> proplists:is_defined(rabbit, Apps)
-    end.
+    rabbit_nodes:is_running(Node, rabbit).
 
 environment() ->
     lists:keysort(
