@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is VMware, Inc.
-%% Copyright (c) 2007-2011 VMware, Inc.  All rights reserved.
+%% Copyright (c) 2007-2012 VMware, Inc.  All rights reserved.
 %%
 
 -module(rabbit_prelaunch).
@@ -22,7 +22,6 @@
 
 -define(BaseApps, [rabbit]).
 -define(ERROR_CODE, 1).
--define(EPMD_TIMEOUT, 30000).
 
 %%----------------------------------------------------------------------------
 %% Specs
@@ -244,16 +243,15 @@ duplicate_node_check([]) ->
     %% Ignore running node while installing windows service
     ok;
 duplicate_node_check(NodeStr) ->
-    Node = rabbit_misc:makenode(NodeStr),
-    {NodeName, NodeHost} = rabbit_misc:nodeparts(Node),
-    case names(NodeHost) of
+    Node = rabbit_nodes:make(NodeStr),
+    {NodeName, NodeHost} = rabbit_nodes:parts(Node),
+    case rabbit_nodes:names(NodeHost) of
         {ok, NamePorts}  ->
             case proplists:is_defined(NodeName, NamePorts) of
                 true -> io:format("node with name ~p "
                                   "already running on ~p~n",
                                   [NodeName, NodeHost]),
-                        [io:format(Fmt ++ "~n", Args) ||
-                            {Fmt, Args} <- rabbit_control:diagnostics(Node)],
+                        io:format(rabbit_nodes:diagnostics([Node]) ++ "~n"),
                         terminate(?ERROR_CODE);
                 false -> ok
             end;
@@ -279,15 +277,3 @@ terminate(Status) ->
                       after infinity -> ok
                       end
     end.
-
-names(Hostname) ->
-    Self = self(),
-    process_flag(trap_exit, true),
-    Pid = spawn_link(fun () -> Self ! {names, net_adm:names(Hostname)} end),
-    timer:exit_after(?EPMD_TIMEOUT, Pid, timeout),
-    Res = receive
-              {names, Names}        -> Names;
-              {'EXIT', Pid, Reason} -> {error, Reason}
-          end,
-    process_flag(trap_exit, false),
-    Res.
